
-- return the window size
function getWindowSize()
	return 556, 246
end

-- return the main font
-- the return is a list of font filename and size options
-- the first one found will be used
function getFont()
	if isMac then
		return { { name="Verdana.ttf", size=13 }, { name="Monaco.dfont", size=13 } }
	else
		return { { name="Verdana.ttf", size=13 }, { name="arial.ttf", size=13 } }
	end
end
-- return the label font
function getLabelFont()
	if isMac then
		return { { name="Verdana.ttf", size=11 }, { name="Monaco.dfont", size=11 } }
	else
		return { { name="Verdana.ttf", size=11 }, { name="arial.ttf", size=11 } }
	end
end
-- return the text field font
function getTextFieldFont()
	if isMac then
		return { { name="Verdana.ttf", size=11 }, { name="Monaco.dfont", size=11 } }
	else
		return { { name="Verdana.ttf", size=11 }, { name="arial.ttf", size=11 } }
	end
end

-- create and lay out all the controls
function createUI()

	-- load textures
	atlas = texture( "buttons.png" )

	-- constants
	local kKnobSize = 36
	local kKnobSpace = 50
	local kTextFieldH = 12
	local kButtonH = 15
	local kDropdownW = 60
	local kLabelSpaceV = 30
	local pad = 10.0
	local ww, wh = getWindowSize()
		
	-- make renderers
	theKnobRenderer = knobRenderer( atlas, 2, 128-2-kKnobSize, kKnobSize, 52, 128-2-kKnobSize )

	-- set window background colour
	setBackgroundColour( 0.5, 0.1, 0.1 )

	-- set location of the banner
	setBannerPosition( wh - 20 )
	setBannerColour( 0.9, 0.9, 0.9, 0.05 )
	setBannerTextColour( 0.6, 0.6, 0.6, 1.0 )

	-- set outline colours for groups
	setGroupActiveColour( 228/255, 95/255, 95/255, 1.0 )
	setGroupInactiveColour( 0.5, 0.5, 0.5, 1.0 )
	
	local function knobAndTextField( group, param, labelText, x, y, w )
		local w = w or kKnobSize
		local size = kKnobSize
		local knobx = x + 0.5 * ( kKnobSize - size ) + 0.5 * ( w - kKnobSize )
		group:add( knob( param, knobx, y+kTextFieldH+1, size ) )
		local tex = textField( param, x, y, w, kTextFieldH )
		group:add( tex )
		local lab = label( labelText, x+0.5*w, y-3 )
		lab:setBackgroundColour( 0.1, 0.1, 0.15, 0.3 )
		lab:setForegroundColour( 0.9, 0.9, 0.9, 1.0 )
		group:add( lab )
	end
	
	local function labelledDropDown( group, param, labelText, x, y, w )
		w = w or kDropdownW
		group:add( dropDown( param, x, y, w, kTextFieldH ) )
		local lab = label( labelText, x+0.5*w, y-3 )
		lab:setBackgroundColour( 0.1, 0.1, 0.15, 0.3 )
		lab:setForegroundColour( 0.9, 0.9, 0.9, 1.0 )
		group:add( lab )
	end

	-- make groups and knobs
	--
	
	local groupPad = { 10, 6, 9, 9 }

	mixGroup = group( "Mix", groupPad )
	knobAndTextField( mixGroup,	"Dry Level", 	"Dry", 0*kKnobSpace, 0 )
	knobAndTextField( mixGroup, "Effect Level", "Wet", 1*kKnobSpace, 0 )

	speedGroup = group( "Speed", groupPad )
	local x = 0
	local y = 0
	knobAndTextField( speedGroup, "Speed (superfine)", 	"S-fine", 	x, y )
	x = x + kKnobSpace
	knobAndTextField( speedGroup, "Speed (fine)", 		"Fine", 		x, y )
	x = x + kKnobSpace
	knobAndTextField( speedGroup, "Speed (coarse)", 	"Coarse", 		x, y )
	x = x + kKnobSize + pad

	depthGroup = group( "Depth", groupPad )
	knobAndTextField( depthGroup, "Depth (fine)", 	"Fine", 	0*kKnobSpace, 0 )
	knobAndTextField( depthGroup, "Depth (coarse)", "Coarse", 	1*kKnobSpace, 0 )

	shapeGroup = group( "Shape", groupPad )
	local extra = 30
	knobAndTextField( shapeGroup, "Shape", 	"Shape", 	0*kKnobSpace, 0, kKnobSize+extra )
	knobAndTextField( shapeGroup, "Spread", "Spread", 	1*kKnobSpace+extra, 0 )

	chorusGroup = group( "Chorus/Flange", groupPad )
	knobAndTextField( chorusGroup, "Bias", 		"Bias", 	0*kKnobSpace, 0 )
	knobAndTextField( chorusGroup, "Feedback", 	"Feedback", 1*kKnobSpace, 0 )

	speedLfoGroup = group( "Speed LFO", groupPad )
	local x = 0
	local y = 0
	knobAndTextField( speedLfoGroup, "Speed LFO Speed", "Speed", x, y )
	x = x + kKnobSpace
	knobAndTextField( speedLfoGroup, "Speed LFO Depth", "Depth", x, y )
	x = x + kKnobSize + pad

	depthLfoGroup = group( "Depth LFO", groupPad )
	local x = 0
	local y = 0
	knobAndTextField( depthLfoGroup, "Depth LFO Speed", "Speed", x, y )
	x = x + kKnobSpace
	knobAndTextField( depthLfoGroup, "Depth LFO Depth", "Depth", x, y )
	x = x + kKnobSize + pad

	oscGroup = group( "OSC", groupPad )
	knobAndTextField( oscGroup, "OSC Port Offset", "Port", 0*kKnobSpace, 0 )

	-- positions!

	xbase = 20
	ybase = 20+10

	xbase1 = xbase 	+ speedGroup:width() 		+ pad
	xbase2 = xbase1 + mixGroup:width() 			+ pad
	xbase3 = xbase2 + shapeGroup:width() 		+ pad
	ybase1 = ybase 	+ speedLfoGroup:height()	+ pad
	ybase2 = ybase1 + speedGroup:height() 		+ pad

	speedLfoGroup:setPosition( xbase, ybase )
	speedGroup:setPosition( xbase, ybase1 )
	depthLfoGroup:setPosition( xbase1, ybase )
	depthGroup:setPosition( xbase1, ybase1 )
	chorusGroup:setPosition( xbase2, ybase )
	shapeGroup:setPosition( xbase2, ybase1 )
	mixGroup:setPosition( xbase3, ybase1 )
	oscGroup:setPosition( xbase3, ybase )

	-- add directly positioned objects
	--

	x = xbase3
	y = ybase2 - 25

	-- add the prefs button
	add( button( "Prefs", "Prefs", x, y, 90, 30 ) )

	-- add the info display object
	x = xbase
	y = y + 10
	add( infoDisplay( x, y, 100, 40 ) )

end		-- of createUI()
